@@ -19,7 +19,6 @@ sub new
     confess "Required param '$_' was not provided"
       unless defined($args{$_});
   }# end foreach()
-  
   return bless \%args, $class;
 }# end new()
 
@@ -38,8 +37,11 @@ sub hook
   
   my $length_received = defined($data) ? length($data) : 0;
   my $context = $s->context;
-  $context->r->pnotes( total_loaded => ($context->r->pnotes('total_loaded') || 0) + $length_received);
-  my $percent_complete = sprintf("%.2f", $context->r->pnotes('total_loaded') / $ENV{CONTENT_LENGTH} * 100 );
+  my $CONTENT_LENGTH = $ENV{CONTENT_LENGTH} || $context->r->pnotes('content_length');
+  my $total_loaded = ($context->r->pnotes('total_loaded') || 0) + $length_received;
+  $context->r->pnotes( total_loaded => $total_loaded);
+  my $percent_complete = sprintf("%.2f", $total_loaded / $CONTENT_LENGTH * 100 );
+#warn "___Total: '$CONTENT_LENGTH' | Loaded: '$total_loaded' | Complete: $percent_complete%\n";
   
   # Mark our start time, so we can make our calculations:
   my $start_time = $context->r->pnotes('upload_start_time');
@@ -49,11 +51,13 @@ sub hook
     $context->r->pnotes('upload_start_time' => $start_time);
   }# end if()
   
+  
+  
   # Calculate elapsed, total expected and remaining time, etc:
   my $elapsed_time        = gettimeofday() - $start_time;
   my $bytes_per_second    = $context->r->pnotes('total_loaded') / $elapsed_time;
   $bytes_per_second       ||= 1;
-  my $total_expected_time = int( ($ENV{CONTENT_LENGTH} - $length_received) / $bytes_per_second );
+  my $total_expected_time = int( ($CONTENT_LENGTH - $length_received) / $bytes_per_second );
   my $time_remaining      = int( (100 - $percent_complete) * $total_expected_time / 100 );
   $time_remaining         = 0 if $time_remaining < 0;
   
@@ -73,6 +77,7 @@ sub hook
   if( ! $did_init )
   {
     $context->r->pnotes( did_init => 1 );
+
     $s->{handler_class}->upload_start( $context, $Upload )
       or return;
     
